@@ -107,11 +107,12 @@ sub verbose_cmd ($)
 }
 
 # Return an array of lines from running $VC diff -u on the named files.
-sub get_diffs ($$)
+# If $PRISTINE, do not honor e.g., git's --ext-diff option.
+sub get_diffs ($$$)
 {
-  my ($vc, $f) = @_;
+  my ($vc, $f, $pristine) = @_;
 
-  my @cmd = ($vc->diff_cmd(), @$f);
+  my @cmd = ($pristine ? $vc->diff_pristine() : $vc->diff_cmd(), @$f);
   $verbose
     and verbose_cmd \@cmd;
   open PIPE, '-|', @cmd
@@ -128,12 +129,13 @@ sub get_diffs ($$)
           . join (' ', @cmd) . "': $PROCESS_STATUS\n";
     }
 
-  # Remove the single space from what would otherwise be empty
-  # lines in unified diff output.
-  foreach my $line (@added_lines)
+  # This may not be needed for git (for it, just run
+  # git config diff.suppress-blank-empty true), but for other
+  # version control systems, it helps to normalize diff output.
+  foreach my $i (0..$#added_lines)
     {
-      $line eq ' '
-        and $line = '';
+      $added_lines[$i] eq ' '
+        and $added_lines[$i] = '';
     }
 
   return \@added_lines;
@@ -144,7 +146,7 @@ sub get_new_changelog_lines ($$)
 {
   my ($vc, $f) = @_;
 
-  my $diff_lines = get_diffs ($vc, [$f]);
+  my $diff_lines = get_diffs ($vc, [$f], 1);
   if (@$diff_lines == 0)
     {
       my $vc_name = $vc->name();
@@ -324,6 +326,7 @@ sub run_command
 # Given the part of a ChangeLog line after a leading "\t* ",
 # return the list of named files.  E.g.,
 # * foo.c: descr
+# * foo.c (func_1, func_2): descr
 # * lib/bar.c (func): descr
 # * glarp.c (struct) [member]: descr
 # and multiple files per line, with each comma-separated entry potentially
@@ -678,6 +681,7 @@ sub check_attribution($$)
       return;
     }
 
+  $$author =~ s/  +</ </;
   $$author eq $name_and_email
     or die "$ME: --author/ChangeLog mismatch:\n  $$author\n  $name_and_email\n";
 }
@@ -781,7 +785,6 @@ sub main
 
   my $vc = VC->new ($changelog_file_name[0]);
   my $vc_name = $vc->name();
-  my @vc_diff = $vc->diff_cmd();
 
   # Key is ChangeLog file name, value is a ref to list of
   # lines added to that file.
@@ -849,28 +852,50 @@ sub main
       # 2006-08-19  Jim Meyering  <jim@meyering.net>
       # Ignore the following one, too, which should be blank.
       my $n_log_lines = @log_lines;
-      if (3 <= $n_log_lines
-          && $log_lines[0] =~ /^\+\d{4}-\d\d-\d\d  (.*)/)
+      if (3 <= $n_log_lines)
         {
-          my $name_and_email = $1;
-          check_attribution $name_and_email, \$author;
-          shift @log_lines;
-
-          # Accept and ignore a second ChangeLog attribution line.  E.g.,
-          # 2006-09-29  user one  <u1@example.org>
-          #         and user two  <u2@example.org>
-          # The "and " on the second line is optional.
-          $log_lines[0] =~ /^\+\t(?:and )?[^<]+<.*>$/
-            and shift @log_lines;
-
-          if ($log_lines[0] ne '+')
+          if ($log_lines[0] =~ /^\+\d{4}-\d\d-\d\d  (.*)/)
             {
-              $log_lines[0] =~ s/^\+//;
-              die "$ME:$log: unexpected, non-blank line after first:\n"
-                . $log_lines[0] . "\n";
+              my $name_and_email = $1;
+              check_attribution $name_and_email, \$author;
+              shift @log_lines;
+
+              # Accept and ignore a second ChangeLog attribution line.  E.g.,
+              # 2006-09-29  user one  <u1@example.org>
+              #         and user two  <u2@example.org>
+              # The "and " on the second line is optional.
+              $log_lines[0] =~ /^\+\t(?:and )?[^<]+<.*>$/
+                and shift @log_lines;
+
+              if ($log_lines[0] ne '+')
+                {
+                  $log_lines[0] =~ s/^\+//;
+                  die "$ME:$log: unexpected, non-blank line after first:\n"
+                    . $log_lines[0] . "\n";
+                }
+              shift @log_lines;
+              $offset += ($n_log_lines - @log_lines);
+            }
+          elsif ($log_lines[@log_lines-2] =~ /^\+\d{4}-\d\d-\d\d  (.*)/
+                 && $log_lines[@log_lines-1] =~ /^\+ ?$/)
+            {
+              # Handle the case in which the latest ChangeLog entry
+              # has a header that is identical to the previous one.
+              # That could result in diff output like this:
+              #
+              # @@ -1,3 +1,7 @@
+              #  2011-03-04  Joe Random  <jr@example.com>
+              #
+              # +       * x: y
+              # +
+              # +2011-03-04  Joe Random  <jr@example.com>
+              # +
+              #
+              # Before the 2011-05-05 fix, vc-dwim would include
+              # that header line at the end of the commit log.
+              pop @log_lines;
+              pop @log_lines;
             }
-          shift @log_lines;
-          $offset += ($n_log_lines - @log_lines);
         }
 
       # FIXME: now that we have this find_author function,
@@ -966,7 +991,7 @@ sub main
 
           if ($line =~ /^\*/)
             {
-              $line =~ /^\* (\S.*?):/
+              $line =~ /^\* (\S.*?(:|\)$))/
                 or die "$ME:$log: line of unexpected form:\n$line";
               my $f_spec = $1;
               foreach my $file (change_log_line_extract_file_list ($f_spec))
@@ -1002,11 +1027,17 @@ sub main
 
   # Collect diffs of non-ChangeLog files.
   # But don't print diff output unless we're sure everything is ok.
-  my $diff_lines = get_diffs $vc, \@affected_files;
+  my $diff_lines = get_diffs $vc, \@affected_files, 1;
 
   cross_check $vc, \@affected_files, $diff_lines;
 
   print join ("\n", @log_msg_lines), "\n";
+
+  # If a user's diff settings may produce non-default-formatted diffs,
+  # then recompute those diffs now, but using their settings (not pristine).
+  $vc->diff_is_pristine
+    or $diff_lines = get_diffs $vc, \@affected_files, 0;
+
   print join ("\n", @$diff_lines), "\n";
 
   # FIXME: add an option to take ChangeLog-style lines from a file,
@@ -1033,6 +1064,11 @@ sub main
             [@changelog_file_name, @affected_files];
         }
     }
+
+  # Warn if the first line of the log starts with "* ".
+  # That indicates a missing one-line summary.
+  $log_msg_lines[0] =~ /^\* /
+    and warn "$ME: $changelog_file_name[0]: no one-line summary\n";
 }
 
 main();
